@@ -24,6 +24,7 @@ THE SOFTWARE.
 
 
 import operator
+import sys
 
 from pytools.decorator import decorator
 
@@ -351,7 +352,7 @@ def single_valued(iterable, equality_pred=operator.eq):
 # }}}
 
 
-# {{{ memoization
+# {{{ memoization / attribute storage
 
 @my_decorator
 def memoize(func, *args):
@@ -370,22 +371,45 @@ def memoize(func, *args):
         result = func(*args)
         func._memoize_dic[args] = result
         return result
+
 FunctionValueCache = memoize
 
 
-@my_decorator
-def memoize_method(method, instance, *args):
+class _HasKwargs(object):
+    pass
+
+def memoize_method(method):
+    """Supports cache deletion via ``method_name.clear_cache(self)``."""
+
     cache_dict_name = intern("_memoize_dic_"+method.__name__)
-    try:
-        return getattr(instance, cache_dict_name)[args]
-    except AttributeError:
-        result = method(instance, *args)
-        setattr(instance, cache_dict_name, {args: result})
-        return result
-    except KeyError:
-        result = method(instance, *args)
-        getattr(instance, cache_dict_name)[args] = result
-        return result
+
+    def wrapper(self, *args, **kwargs):
+        if kwargs:
+            key = (_HasKwargs, frozenset(kwargs.iteritems())) + args
+        else:
+            key = args
+
+        try:
+            return getattr(self, cache_dict_name)[key]
+        except AttributeError:
+            result = method(self, *args, **kwargs)
+            setattr(self, cache_dict_name, {key: result})
+            return result
+        except KeyError:
+            result = method(self, *args, **kwargs)
+            getattr(self, cache_dict_name)[key] = result
+            return result
+
+    def clear_cache(self):
+        delattr(self, cache_dict_name)
+
+    if sys.version_info >= (2, 5):
+        from functools import update_wrapper
+        new_wrapper = update_wrapper(wrapper, method)
+        new_wrapper.clear_cache = clear_cache
+
+    return new_wrapper
+
 
 
 def memoize_method_nested(inner):
@@ -424,8 +448,6 @@ def memoize_method_nested(inner):
 
     return new_inner
 
-FunctionValueCache = memoize
-
 # }}}
 
 
@@ -1266,9 +1288,9 @@ class CPyUserInterface(object):
         import os
         for arg in argv[1:]:
             if os.access(arg, os.F_OK):
-                exec open(arg, "r") in execenv
+                exec(compile(open(arg, "r"), arg, 'exec'), execenv)
             else:
-                exec arg in execenv
+                exec(compile(arg, "<command line>", 'exec'), execenv)
 
         # check if the user set invalid keys
         for added_key in (
